Program NetStats;

{
   Reads a FidoNet NODELIST and outputs a list of stats about the nodelist.
}

Uses
    DOS,
    CRT,
    Printer;

CONST
     Version = 'NetStats 3.2.1 2022/10/31';
{ 2018/02/16 vbc - 3.1.1 Changed date format to unix (ccyy/mm/dd) from USA. }
{ 2019/01/10 vbc - 3.1.2 Changed comments regard where to obtain the latest version
                         as original programmer has left Fido }
{ 2022/10/30 vbc - 3.2   Output files now Netstats.rpt (instead of .OUT.
                         Changed where to get it from etc }
{ 2022/10/31 vbc - 3.2.1 Removed any baud rate zero totals from printing.}

TYPE
    Entry     = Record  { these may be useful while working in the pgm }
                  Kind        : string;
                  Number      : String;
                  BbsName     : string;
                  City_ST     : string;
                  Sysop_Name  : string;
                  FonNumbr : string;
                  BaudRate    : String;
                  Flags       : string;
                end;

    NodeItem  = record
                   Net        : word;
                   Node       : word;
                   FonNumbr: String[10];
                end;

{ Var   section  }
VAR
   NetNumber : word;
   Buf                                           : Array[1..20480] of Char;

   Code,
   k,
   n    : integer;

   { counters ... }

   IfnaNum, i, AdminOVHD, NonComments          : LongInt;
   ZoneCount, RegionCount, HostCount, HubCount, PvtCount, DownCount : LongInt;
   HoldCount, Baud9600, Baud2400, Baud1200, Baud300                 : LongInt;
   CrashMail, MailOnly, HST, PEP, HAY, MAX , H96                    : LongInt;

   V32,V32b,V42,V42b,V33                                            : LongInt;


   ParmCount  : byte;

   { handles.....}
   OutF,           { will be the output file handle }
   INodelist  : text;  { St. Louis format nodelist file }

   { file name strings ....}
   IfnaList,
   OutFile    : string;

   ALine      : string;

   Parts      : array[1..8] of string;

   AComment   : Boolean;

   Lines      : Entry;

   Apoint     : NodeItem;

   FTIME      : DateTime;

   FYear,FMonth,FDay :Integer;
   FKb : real;

{ Label section  }
LABEL
    GetFileNames,
    GotFileNames;

{ ------ Procedure Definitions ------- }
Function PERCENTS (Num1,Num2:longint) :Real;
Begin
  Percents := 100*(num1/num2);
End;

PROCEDURE ParseLines(OneLine:string; var LineParts: Entry ;Var Comment:Boolean);
Var
   i,
   k      : integer;
   c      : string[1];
   Pieces : array[1..8] of string;

begin
     Pieces[1] := '';
     Pieces[2] := '';
     Pieces[3] := '';
     Pieces[4] := '';
     Pieces[5] := '';
     Pieces[6] := '';
     Pieces[7] := '';
     Pieces[8] := '';

     LineParts.Kind        := 'xxx';
     LineParts.Number      := '';
     LineParts.BbsName     := '';
     LineParts.City_ST     := '';
     LineParts.Sysop_Name  := '';
     LineParts.FonNumbr    := '';
     LineParts.BaudRate    := '';
     LineParts.Flags       := '';


   k:= POS(';',OneLine);
   If ( (k > 0) OR ( Length(OneLine) < 2) ) then
   begin
         Comment := True;
         Exit;
   end;

   i := 0;
   k := 1;

   while i < Length(OneLine)  do
   begin
      inc(i);
      if (OneLine[i] <> ',') OR ( k = 8) then
         Pieces[k] := Pieces[k] + OneLine[i]
      else
         inc(k);
   end;

     LineParts.Kind        := Pieces[1];
     LineParts.Number      := Pieces[2];
     LineParts.BbsName     := Pieces[3];
     LineParts.City_ST     := Pieces[4];
     LineParts.Sysop_Name  := Pieces[5];
     LineParts.FonNumbr    := Pieces[6];
     LineParts.BaudRate    := Pieces[7];
     LineParts.Flags       := Pieces[8];

     Comment := False;
end; { of the Procedure ParseLines }
{--------------------------------------------------------------------}

PROCEDURE STDIO;
{ the following allows redirection of std i/o in TP4 or TP5 }
{  WILL DISABLE USE OF COLORS ! }
Begin
        Assign(Input,'');
        Reset (Input);

        Assign (Output,'');
        Rewrite(Output);
End; { of PROCEDURE STDIO }

PROCEDURE StripOut(What: string; VAR Target:string);
Begin
                While Pos(What,Target) > 0 do
                  Target[Pos(What,Target)] := ' ';
End;

Procedure Show_Em(DevName:String);
Begin

 If DevName = '' then
    DevName := 'CON';
 If DevName <> 'CON' then WriteLn(' ');  WriteLn(' Sending output to file/device : ',DevName);
 Assign(OutF,DevName);
 ReWrite(OutF);
 WriteLn(OutF,'                    ',Version);
 WriteLn(OutF);
 WriteLn(OutF, ' Input Nodelist file      : ', IfnaList);
 WriteLn(OutF, ' File has a Date Stamp of : ',FYear:4,'/',FMonth,'/',FDay );
 WriteLn(OutF, ' File Size (uncompressed) : ',FKb:9:2,' KBytes.');
 WriteLn(OutF);
 WriteLn(OutF,' The Input Nodelist file  has ',i,' Nodes in it');
 WriteLn(OutF,'  and a total of ',NonComments:6,' non-comment entries.');
 If DevName <> 'CON' then WriteLn(OutF);
 WriteLn(OutF,' The list has ',ZoneCount:5,' Zones listed.');
 WriteLn(OutF,'              ',RegionCount:5,' Region Coordinators listed.');
 WriteLn(OutF,'              ',HostCount:5, ' Network Hosts listed.');
 WriteLn(OutF,'              ',HubCount:5,' Hubs listed.');
 WriteLn(OutF,'              ',DownCount:5,' DOWN Nodes listed.');
 WriteLn(OutF,'              ',PvtCount:5,' PRIVATE Nodes listed.');
 WriteLn(OutF,'              ',HoldCount:5,' HOLD Nodes listed.');
 If DevName <> 'CON' then  WriteLn(OutF);
 WriteLn(OutF,' Administrative overhead = ',AdminOVHD,' (',PERCENTS(AdminOVHD,i):6:2,' %) ');
 WriteLn(OutF,' Private Nodes overhead  = ',PvtCount ,' (',PERCENTS(PvtCount,i):6:2,' %)');
 If DevName <> 'CON' then  WriteLn(OutF);
 WriteLn(OutF,' Baud Rate Summary :');
 WriteLn(OutF,'              9600 = ',Baud9600:6,' ( ', PERCENTS(Baud9600,i):6:2,' %)');
 If HST <> 0 then
 WriteLn(OutF,'                         (HST''s  =',HST:5,' or ',PERCENTS(HST,Baud9600):6:2,' % of the 9600 baud modems.)');
 If PEP <> 0 then
 WriteLn(OutF,'                         (PEP''s  =',PEP:5,' or ',PERCENTS(PEP,Baud9600):6:2,' % "        "    "     "   )');
 If MAX <> 0 then
 WriteLn(OutF,'                         (MAX''s  =',MAX:5,' or ',PERCENTS(MAX,Baud9600):6:2,' % "        "    "     "   )');
 If HAY <> 0 then
 WriteLn(OutF,'                         (HAY''s  =',HAY:5,' or ',PERCENTS(HAY,Baud9600):6:2,' % "        "    "     "   )');
 WriteLn(OutF,'                         (V32''s  =',V32:5,' or ',PERCENTS(V32,Baud9600):6:2,' % "        "    "     "   )');
 WriteLn(OutF,'                         (V32B''s =',V32B:5,' or ',PERCENTS(V32B,Baud9600):6:2,' % "        "    "     "   )');
 WriteLn(OutF,'                         (V42''s  =',V42:5,' or ',PERCENTS(V42,Baud9600):6:2,' % "        "    "     "   )');
 WriteLn(OutF,'                         (V42B''s =',V42B:5,' or ',PERCENTS(V42B,Baud9600):6:2,' % "        "    "     "   )');
 If Baud2400 <> 0 then
 WriteLn(OutF,'              2400 = ',Baud2400:6,' (', PERCENTS(Baud2400 ,i):6:2,' %)');
 If Baud1200 <> 0 then
 WriteLn(OutF,'              1200 = ',Baud1200:6,' (', PERCENTS(Baud1200 ,i):6:2,' %)');
 WriteLn(OutF,'               300 = ',Baud300:6,' (', PERCENTS(Baud300 ,i):6:2,' %)');
 If DevName <> 'CON' then  WriteLn(OutF);
 WriteLn(OutF,' CrashMail capable = ',CrashMail:6,' (', PERCENTS(CrashMail ,i):6:2,' %)');
 WriteLn(OutF,' MailOnly Nodes    = ',MailOnly:6,' (', PERCENTS(MailOnly ,i):6:2,' %)');
 If DevName <> 'CON' then  WriteLn(OutF);
 WriteLn(OutF,' [ Report produced by Netstats - A P.D. program, now available from 2:250/1 ]');
 Flush (OutF);
 Close (OutF);
End;
{--------------------------------------------------------------------}

Procedure GetNewList(Var ListName:PathStr);
Var ListDir  : DirStr;
    ListFile : NameStr;
    ListExt  : ExtStr;
    SearchData : SearchRec;
    CheckUse   : longInt;
    CheckName  : String;

Begin
  ListName := FExpand(ListName);
  FSplit(ListName,ListDir,ListFile,ListExt);
  ListName := ListDir + ListFile + '.*';
  FindFirst(ListName,Archive,SearchData);
  If DOSERROR <> 0 then
  Begin
    Writeln('OOPS!  The Selected NodeList file does not exist, aborting.');
    Halt(1);
  End;
  CheckUse := 0;
  WriteLn(' Obtaining the latest Nodelist File for you....');
  Repeat
    If (SearchData.Time > CheckUse)
                AND
    (SearchData.Name[Length(SearchData.Name)] in ['1','2','3','4','5','6','7','8','9','0'])    then
      Begin
        CheckUse := SearchData.Time;
        UnPackTime(CheckUse,FTIME);
        With FTIME
          Do
            begin
              FYear  := FTIME.Year;
              FMonth := FTIME.Month;
              FDay   := FTIME.Day;
            end;
        Fkb    := SearchData.Size/1024;
        CheckName := SearchData.Name;
        WriteLn('File Match Found: ', CheckName,'  ', FYear,'/',Fmonth,'/',Fday,' ',FKb:7:1,' KBytes.');
      End;
    FindNext(SearchData);
  Until DOSERROR = 18;
  ListName := ListDir + CheckName;
End;

{--------------------------------------------------------------------}

FUNCTION Exist(fname:string): Boolean;
{ Tests for existence of a file - will match wildcards, too }
Var
   Result : Boolean;
   DirInfo: SearchRec;
   code   : integer;

begin { this function }
     Result := False;
     FindFirst(fname,AnyFile,DirInfo);
     code := DosError;
     case code of
          0:    begin
                  Result := True;
                end;
          2,3:  begin
                  Result := False;
                 { WriteLn(' No Such File/Path/Directory Found'); }
                end;
         18:    begin
                  Result := False;
                end;
     else
         begin
              Result := False;
              WriteLn;
              WriteLn(' Undefined error code ',code,' has been encountered.');
         end;
     end;
{ writeln(code); }
Exist := Result;
End; { of Function EXIST }
{ ----------------------------------------------------------------- }

PROCEDURE ItsTime(Fname:String);
Var
   DirInfo: SearchRec;
   code   : integer;
begin { this function }
     FindFirst(fname,AnyFile,DirInfo);
     code := DosError;
     case code of
          0:    begin
                  UnPackTime(DirInfo.Time,FTIME);
                  With FTIME
                    Do
                      begin
                        FYear  := FTIME.Year;
                        FMonth := FTIME.Month;
                        FDay   := FTIME.Day;
                      end;
                        Fkb    := DirInfo.Size/1024;
                end;
          2,3:  begin
                  WriteLn(' No Such File/Path/Directory Found');
                end;
         18:    begin
                  WriteLn(' Unusual Error Code: 18');
                end;
     else
         begin
              WriteLn;
              WriteLn(' Undefined error code ',code,' has been encountered.');
         end;
     end;

End;
{ ----------------------------------------------------------------- }

Procedure DataFile;

CONST
  Q='"';
  C=',';

Var OutFile : text;
Begin

 Assign(OutFile,'NETSTATS.DAT');

 if Exist ('NETSTATS.DAT')
 then
   Append(OutFile)
 else
   Rewrite(OutFile);

   Write(OutFile,

   FKb:7:1,C,
   FMonth,C,
   FDay,C,
   FYear,C,
   Q,IfnaList,Q,C,
   i,C,
   NonComments,C,

   ZoneCount,C,
   RegionCount,C,
   HostCount,C,
   HubCount,C,
   DownCount,C,
   PvtCount,C,
   HoldCount,C,
   AdminOVHD,C,

   Baud9600,C,
   HST,C,
   PEP,C,
   MAX,C,
   HAY,C,
   V32,C,
   V32B,C,
   V42,C,
   V42B,C,
   Baud2400,C,
   Baud1200,C,
   Baud300,C,

   CrashMail,C,
   MailOnly );
   WriteLn(OutFile);

   Flush(OutFile);
   Close(OutFile);
   Writeln;
   WriteLn(' Output data also as NETSTATS.DAT in comma-delimited format.')
end;

{--------------------------------------------------------------------}

BEGIN { Main Program Starts here }

ClrScr;

WriteLn('                   ',Version);
WriteLn;
writeln( ' This program is placed into the Public Domain. Source code is now available');
writeln( ' by file requesting netstats.zip from FidoNet 2:250/1 .');
writeln( ' or via vbcoen@gmail.com ');
Writeln;
writeln( ' Vincent Coen, Sysop at 2:250/1');
writeln;

TextColor(7);
TextBackground(0);
LowVideo;

   ParmCount := ParamCount;
   Case ParmCount of
     0:
       begin
            Writeln( ' You may enter the NODELIST file as');
            WriteLn( ' command-line Parameters. Example:');
            WriteLn('NetStats c:\opus\nodelist\Zone1.323');
            WriteLn(' uses the nodelist ZONE1.323 as input from the');
            WriteLn(' specified subdirectory and checks for duplicates in');
            WriteLn(' that nodelist. The output file is ALWAYS called:' );
            WriteLn(' NetStats.OUT .');
            WriteLn;
            WriteLn(' Since you didn''t give a parameter, you will now');
            WriteLn(' be prompted for the NODELIST name..... ');
            Goto GetFileNames;
       end;
      1:
         begin
              IfnaList := ParamStr(1);
              GetNewList(IfnaList);
              Goto GotFileNames;
         end;
       Else
           begin
             WriteLn(' Error in number of command line parameters.');
             WriteLn(' Halting program....');
             Halt(3);
           end;
       End;

GetFileNames:
   WriteLn;
   WriteLn(' What is the Nodelist INPUT file to be used ? ');
   ReadLn(IfnaList);
GotFileNames:
   Writeln;
   WriteLn(' (Using ',IfnaList,' as the INPUT File.)');
   WriteLn(' (Using NetStats.rpt  as the OUTPUT File.)');

   { Open the IFNA nodelist file }
      If EXIST(IfnaList)
      then
        begin
          Assign(INodeList,IfnaList);
          ItsTime(IfnaList);
        end
      else
        Begin
          WriteLn(' The requested file: ',IfnaList, ' does not exist!');
          Halt(2);
        End;
      SetTextBuf(INodeList,Buf);  { reads that sucker 20K at a time, eh? }
      Reset(INodeList);
      writeln (IfnaList,' OPEN''d for examination....');

      i := 0; { init the counter }
      ZoneCount    := i;
      RegionCount  := i;
      HostCount    := i;
      HubCount     := i;
      PvtCount     := i;
      DownCount    := i;
      HoldCount    := i;
      Baud9600     := i;
      Baud2400     := i;
      Baud1200     := i;
      Baud300      := i;
      CrashMail    := i;
      MailOnly     := i;
      HST          := i;
      PEP          := i;
      HAY          := i;
      H96          := i;
      MAX          := i;
      v32          := i;
      v32b         := i;
      v42          := i;
      v42b         := i;
      v33          := i;
      NonComments  := i;

      While NOT Eof(INodeList) do
      begin
         ReadLn(INodelist,Aline); { get a node from FidoNet List }
         ParseLines(Aline,Lines,AComment);

         If NOT AComment then
         Begin

           Inc(NonComments);
           If Lines.Kind = 'Zone' then
           begin
              Val(Lines.Number,NetNumber,Code);
              WriteLn;
              WriteLn;
              HighVideo;
              TextColor(12);
              WriteLn('Zone ',NetNumber,'  (',Lines.City_ST,')');
              TextColor(7);
              LowVideo;
              Inc(ZoneCount);
           end;

           If Lines.Kind = 'Region' then
           begin
              Val(Lines.Number,NetNumber,Code);
              TextColor(14);
              WriteLn;
              WriteLn('Region ',NetNumber,'  (',Lines.City_ST,')');
              TextColor(7);
              Inc(RegionCount);
           end;

           If Lines.Kind = 'Host' then
           begin
              Val(Lines.Number,NetNumber,Code);
              If WhereX > 70 then WriteLn;
              TextColor(9);
              Write( ' Net ',NetNumber);
              TextColor(7);
             Inc(HostCount);
           end;

           If Lines.Kind = 'Hub' then
               Inc(HubCount);

           If Lines.Kind = 'Down' then
               Inc(DownCount);

           If Lines.Kind = 'Pvt' then
              Inc(PvtCount);

           If Lines.Kind = 'Hold' then
           Inc(HoldCount);

           If Lines.Kind = '' then
           Begin

             Inc(i);

             { WriteLn(LST,Lines.Number); }

             If  (Lines.BaudRate = '9600')  then
             begin
                Inc(Baud9600);
                If (POS('HST',Lines.Flags)) > 0 then Inc(HST);
                If (POS('PEP',Lines.Flags)) > 0 then Inc(PEP);
                If (POS('H96',Lines.Flags)) > 0 then Inc(HAY);
                If (POS('MAX',Lines.Flags)) > 0 then Inc(MAX);
             end;

             If  (Lines.BaudRate = '2400') then
                Inc(Baud2400);

             If (Lines.BaudRate = '1200') then
                Inc(Baud1200);

             If (Lines.BaudRate = '300')  then
                Inc(Baud300);

             If (POS('CM',Lines.Flags)) > 0 then Inc(CrashMail);
             If (POS('MO',Lines.Flags)) > 0 then Inc(MailOnly);

             If (POS('V32',Lines.Flags)) > 0  then Inc(V32);
             If (POS('V32b',Lines.Flags)) > 0 then Inc(V32b);
             If (POS('V42',Lines.Flags)) > 0  then Inc(V42);
             If (POS('V42b',Lines.Flags)) > 0 then Inc(V42b);
             If (POS('V33',Lines.Flags)) > 0  then Inc(V33);

           End;
         End;
      end;
      IfnaNum := i;
      AdminOVHD := ZoneCount+RegionCount+HostCount+HubCount;
      Close(INodeList);

      V32 := V32 - V32b;  { back out the bis' }
      V42 := V42 - V42b;  { back out the bis' }

 TextColor(7);
 TextBackground(0);
 NormVideo;

 { Output to a file }

   Show_Em('NetStats.rpt');

 { Output to CRT, too }
   Clrscr;
   Show_Em('');

 { Uncomment the following for printed output }
{
   Show_Em('PRN');
 }
 DataFile;
END.  { Main Program ends here   }
